#!/bin/bash

#
# script to submit job or run the vHLLE+UrQMD hybrid
#

jobs=1  # 200
ncores=1  # change ntasks and constraints appropriately

prefix_urqmd=rhic077-2030.Rt14.Rz05.etas02.1PT
prefix=rhic077-2030.Rt14.Rz05.etas02.1PT

#scratch=/scratch/hfftheo/karpenko/hybrid
scratch=.

cd ..

mkdir -p urqmd.out/$prefix_urqmd
mkdir -p urqmd.in
mkdir -p hydro.in
mkdir -p escape5.in

if [ -d hydro.out ]; then
    cd hydro.out
    mkdir -p $prefix
    cd ..
else
    echo "hydro.out does not exist; exiting"
    exit
fi

if [ -d escape5.out ]; then
    cd escape5.out
    mkdir -p $prefix
    cd ..
else
    echo "escape5.out does not exist; exiting"
    exit
fi

for j in `seq 1 $jobs`;
do
    job_file=jobs/$prefix.$j.job
    echo "#!/bin/bash" > $job_file
    echo "#SBATCH --partition=parallel" >> $job_file
    # echo "#SBATCH --constraint=dual" >> $job_file
    # echo "#SBATCH --ntasks=24" >> $job_file
    # echo "#SBATCH --cpus-per-task=1" >> $job_file
    # echo "#SBATCH --mem-per-cpu=5000" >> $job_file
    # echo "#SBATCH --tmp=20000" >> $job_file
    # echo "#SBATCH --job-name=19nc" >> $job_file
    # echo "#SBATCH --time=0-23:00:00" >> $job_file
    # echo "source /home/fias/karpenko/root/bin/thisroot.sh" >> $job_file
    echo "" >> $job_file
    echo "cd `pwd`" >> $job_file

    #----------- urqmd part ---------------
    for k in `seq 1 $ncores`;
    do
	let i=$j*$ncores+$k
	# -- temporary hydro output in the same filesystem
	echo 'mkdir -p hydro.out/$prefix/$i' >> $job_file
	# -- or in the /local space of a computational node
	#echo 'mkdir -p /local/$SLURM_JOB_ID/'"$i" >> $job_file
	#echo 'ln -s /local/$SLURM_JOB_ID/'"$i hydro.out/$prefix/$i" >> $job_file

	echo "pro 197 79" > urqmd.in/$prefix_urqmd.$i
	echo "tar 197 79" >> urqmd.in/$prefix_urqmd.$i
	echo "nev 1" >> urqmd.in/$prefix_urqmd.$i
	echo "tim 200 200" >> urqmd.in/$prefix_urqmd.$i
	echo "ecm 7.7" >> urqmd.in/$prefix_urqmd.$i     # sqrt{s}
	echo "IMP 6.7 8.2" >> urqmd.in/$prefix_urqmd.$i  # impact parameter range
	echo "eos  0" >> urqmd.in/$prefix_urqmd.$i
	echo "ctp 63 3.2" >> urqmd.in/$prefix_urqmd.$i # PARAMETER tau_0
	echo "cto 57 1" >> urqmd.in/$prefix_urqmd.$i
	echo "f13" >> urqmd.in/$prefix_urqmd.$i
	echo "#f14" >> urqmd.in/$prefix_urqmd.$i
	echo "f15" >> urqmd.in/$prefix_urqmd.$i
	echo "f16" >> urqmd.in/$prefix_urqmd.$i
	echo "f18" >> urqmd.in/$prefix_urqmd.$i
	echo "f19" >> urqmd.in/$prefix_urqmd.$i
	echo "f20" >> urqmd.in/$prefix_urqmd.$i
	#seed=`expr $RANDOM + $i`
	echo "rsd $RANDOM" >> urqmd.in/$prefix_urqmd.$i
	echo "xxx" >> urqmd.in/$prefix_urqmd.$i
	echo "" >> urqmd.in/$prefix_urqmd.$i

	bashfile=jobs/$prefix.$i.bash
	hydro_log=hydrologs/$prefix.$i.log
	echo "source /opt/ROOT5/bin/thisroot.sh" > $bashfile
	echo "export ftn09=urqmd.in/$prefix_urqmd.$i" >> $bashfile
	echo "export ftn13=urqmd.out/$prefix_urqmd/f13" >> $bashfile
	echo "export ftn14=urqmd.out/$prefix_urqmd/f14.$i" >> $bashfile
	echo "export ftn15=urqmd.out/$prefix_urqmd/f15" >> $bashfile
	echo "export ftn16=urqmd.out/$prefix_urqmd/f16" >> $bashfile
	echo "export ftn18=urqmd.out/$prefix_urqmd/f18" >> $bashfile
	echo "export ftn19=urqmd.out/$prefix_urqmd/f19" >> $bashfile
	echo "export ftn20=urqmd.out/$prefix_urqmd/f20" >> $bashfile
	echo "time ../urqmd_ic_etatau/urqmd.x86_64" >> $bashfile
	
	#---------- hydro part ---------------
	
	hydro_in=hydro.in/$prefix.$i
	#mkdir -p hydro.out/$prefix/$i
	
	echo "outputDir       hydro.out/$prefix/$i" > $hydro_in 
	echo "" >> $hydro_in
	echo "! EoS type: 0 = complex EoS, 1 = simple table (eosFile)" >> $hydro_in
	echo "eosType       2" >> $hydro_in    # equation of state: 1=Chiral EoS, 2=1st order PT EoS
	echo "eosFile         Laine_nf3.dat" >> $hydro_in
	echo "chiBfile        chiB.txt" >> $hydro_in
	echo "chiSfile        chiS.txt" >> $hydro_in
	echo "T_ch          0.165" >> $hydro_in
	echo "mu_b          0.0285" >> $hydro_in
	echo "mu_q         -0.001" >> $hydro_in
	echo "mu_s          0.007" >> $hydro_in
	echo "gammaS        0.935" >> $hydro_in
	echo "gammaFactor   0.75" >> $hydro_in
	echo "exclVolume    0.0" >> $hydro_in
	echo "" >> $hydro_in
	echo "etaS      0.2" >> $hydro_in   # PARAMETER eta/s
	echo "zetaS     0.0" >> $hydro_in
	echo "e_crit    0.5" >> $hydro_in
	echo "" >> $hydro_in
	echo "nx        161" >> $hydro_in
	echo "ny        161" >> $hydro_in
	echo "nz        101" >> $hydro_in
	echo "xmin     -16.0" >> $hydro_in
	echo "xmax      16.0" >> $hydro_in
	echo "ymin     -16.0" >> $hydro_in
	echo "ymax      16.0" >> $hydro_in
	echo "etamin   -2.5" >> $hydro_in
	echo "etamax    2.5" >> $hydro_in
	echo "" >> $hydro_in
	echo "! icModel: 1=Glauber, 2=input from file,  3= UrQMD IC (file)" >> $hydro_in
	echo "icModel    3" >> $hydro_in
	echo "glauberVar 1" >> $hydro_in
	echo "icInputFile urqmd.out/$prefix_urqmd/f14.$i" >> $hydro_in
	echo "s0ScaleFactor  53.55" >> $hydro_in
	echo "epsilon0   30.0" >> $hydro_in
	echo "impactPar  0.0" >> $hydro_in
	echo "Rg      1.4" >> $hydro_in       # PARAMETER  R_\perp
	echo "Rgz     0.5" >> $hydro_in       # PARAMETER  R_\eta
	echo "tau0       3.2" >> $hydro_in   # PARAMETER tau_0
	echo "tauMax     18.0" >> $hydro_in
	echo "dtau       0.05" >> $hydro_in

	echo "time ../vhlle/hlle_visc $hydro_in > $hydro_log" >> $bashfile

	#------------- escape5 part ---------------------

	escape5_in=escape5.in/$prefix.$i

	echo "surface  hydro.out/$prefix/$i/freezeout.dat" > $escape5_in
	echo "spectra_dir      $scratch/escape5.out/$prefix" >> $escape5_in
	echo "fmax_dir         mult/may2013/rhic2030.tau035.ideal" >> $escape5_in
	echo "number_of_events 1000" >> $escape5_in    # oversampling factor
	echo "rescatter        1" >> $escape5_in    # UrQMD rescattering (1=on)
	echo "weakContribution 0" >> $escape5_in
	echo "shear            1" >> $escape5_in    # delta f viscous correction, Cooper-Frye
	echo "ecrit            0.5" >> $escape5_in
	echo "Nbins            4" >> $escape5_in
	echo "q_max            0.08" >> $escape5_in

	echo "time ../escape5/escape5.visc events $i $escape5_in >> $hydro_log" >> $bashfile
	echo "cd hydro.out/$prefix ; rm -rf $i; cd ../.." >> $bashfile
	echo "bash $bashfile &" >> $job_file
    done
    echo "wait" >> $job_file
    bash $job_file
    #sbatch $job_file
done
